package org.jboss.soa.qa.modeshape;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.ServiceLoader;
import java.util.concurrent.CountDownLatch;

import javax.jcr.Binary;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.Property;
import javax.jcr.PropertyIterator;
import javax.jcr.Repository;
import javax.jcr.RepositoryFactory;
import javax.jcr.Session;
import javax.jcr.observation.Event;
import javax.jcr.observation.EventListener;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

public class WSDLServlet extends HttpServlet 
{
	private static final long serialVersionUID = 1L;
	private static final String CHILD_NODE = "/sequenced/cnd/test.cnd";
	private static final String PARENT_NODE = "/sequenced/cnd";
	
	private Session sess = null;
	
	@Override
	public void doGet(HttpServletRequest request, HttpServletResponse response)
		throws IOException, ServletException
	{				
		String repository = request.getParameter("repository");
		String workspace  = request.getParameter("workspace");
		CountDownLatch latch = new CountDownLatch(1);
		System.out.println("Repository: " + repository + ", workspace: "+ workspace);
		EventListener listener = new NodeAddedListener(latch, CHILD_NODE);
				
		try {
			System.out.println("Retrieving session for repo: " + repository+ " and workspace: " + workspace);
			sess = getSession(getRepository(repository), workspace);
			System.out.println("Registering listener for path " + PARENT_NODE);
			registerListener(listener, PARENT_NODE);
			System.out.println("Publishing file..");
			publish("test.cnd");
			System.out.println("Saving session..");
			sess.save();						
			latch.await();
			
			print("/test.cnd");
		} catch (Exception e)
		{
			e.printStackTrace();
		} finally {
			System.out.println("logout() on session in finally..");
			closeSession(sess);
		}
	}
	
	private void closeSession(Session session) 
	{
		try {
			
			if (session != null)
			{
				session.logout();
			}
		} finally {
			session = null;
		}
		
	}

	private void registerListener(EventListener listener, String node) throws Exception
	{
		if (sess != null)
		{
			System.out.println("Registering listener..");
			sess.getWorkspace().getObservationManager().addEventListener(listener, Event.NODE_ADDED, PARENT_NODE, true, null, null, false);
		}
	}
	
	private void publish(String file) throws Exception
	{
		Node root     = sess.getRootNode();
		Node files    = null;
		Node testFile = null;
		Node content  = null;		
		if (root.hasNode("files"))
		{			
			files = root.getNode("files");
			testFile = files.addNode(file, "nt:file");
			content  = testFile.addNode("jcr:content", "nt:resource"); 
			Binary data = sess.getValueFactory().createBinary(getClass().getResourceAsStream("/" + file));
			content.setProperty("jcr:data", data);
		}
	}
	
	private Repository getRepository(String name) throws Exception
	{
		Repository repository = null;
		Map<String, String> parameters = new HashMap<String, String>();
		parameters.put("org.modeshape.jcr.URL", "jndi:jcr/local?repositoryName="+ name);
		for (RepositoryFactory aFactory : ServiceLoader.load(RepositoryFactory.class)) 
		{
			repository = aFactory.getRepository(parameters);
			if (repository != null) return repository;
		}
		return repository;
	}
	
	private Session getSession(Repository repository, String workspace) throws Exception
	{
		return repository.login(workspace);
	}
	
	private void print(String node) throws Exception
	{
		final String sequence = "sequenced/cnd"; 
		Node root = sess.getRootNode();
		if (root.hasNode(sequence))
		{
			System.out.println(sequence + " found");
			Node parent = root.getNode(sequence + node);
			if (parent.hasProperties())
			{
				System.out.println("Parent node has these properties:");
				PropertyIterator pi = parent.getProperties();
				while (pi.hasNext())
				{
					Property prop = pi.nextProperty();
					if (prop.isMultiple())
					{
						System.out.println("\t multivalued property, skipping it");
					} else {						
						System.out.println(prop.getName() + " = " + prop.getValue().getString());
					}
					
				}
			}
			if (parent.hasNodes())
			{
				System.out.println("Parent has these child nodes:");
				NodeIterator ni = parent.getNodes();
				while (ni.hasNext())
				{
					System.out.println("Node: " + ni.nextNode());
				}
			} else {
				System.out.println("No child nodes found..");
			}
		}
	}
	
}
